@@ -57,7 +57,10 @@ class CppExtractor(BaseExtractor):
         file_path: Path,
         source_code: str,
     ) -> ParseResult:
-        """Extract entities and relations from C++ AST."""
+        """Extract entities and relations from C/C++ AST."""
+        # Set source bytes for correct byte offset handling
+        self._set_source(source_code)
+
         entities: list[Entity] = []
         relations: list[Relation] = []
 
