@@ -59,6 +59,9 @@ class CSharpExtractor(BaseExtractor):
         source_code: str,
     ) -> ParseResult:
         """Extract entities and relations from C# AST."""
+        # Set source bytes for correct byte offset handling
+        self._set_source(source_code)
+
         entities: list[Entity] = []
         relations: list[Relation] = []
 
