@@ -55,6 +55,9 @@ class PythonExtractor(BaseExtractor):
         source_code: str,
     ) -> ParseResult:
         """Extract entities and relations from Python AST."""
+        # Set source bytes for correct byte offset handling
+        self._set_source(source_code)
+
         entities: list[Entity] = []
         relations: list[Relation] = []
 
